@@ -1,5 +1,6 @@
 /* keylist.c
- * Copyright (C) 1998,1999,2000,2001,2002,2003 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003,
+ *               2004 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -40,6 +41,7 @@
 
 static void list_all(int);
 static void list_one( STRLIST names, int secret);
+static void print_card_serialno (PKT_secret_key *sk);
 
 struct sig_stats
 {
@@ -59,23 +61,38 @@ public_key_list( STRLIST list )
 {
   if(opt.with_colons)
     {
-      byte trust_model;
+      byte trust_model,marginals,completes,cert_depth;
       ulong created,nextcheck;
 
-      read_trust_options(&trust_model,&created,&nextcheck);
+      read_trust_options(&trust_model,&created,&nextcheck,
+			 &marginals,&completes,&cert_depth);
 
       printf("tru:");
 
       if(nextcheck && nextcheck <= make_timestamp())
 	printf("o");
-      if(trust_model!=0)
+      if(trust_model!=opt.trust_model)
 	printf("t");
+      if(opt.trust_model==TM_PGP || opt.trust_model==TM_CLASSIC)
+	{
+	  if(marginals!=opt.marginals_needed)
+	    printf("m");
+	  if(completes!=opt.completes_needed)
+	    printf("c");
+	  if(cert_depth!=opt.max_cert_depth)
+	    printf("d");
+	}
 
-      /* We don't show marginals, completes, or cert_depth since
-	 they are not accurate in 1.2.x - they are not updated when
-	 the trustdb is rebuilt with different options. */
+      printf(":%d:%lu:%lu",trust_model,created,nextcheck);
 
-      printf(":%d:%lu:%lu\n",trust_model,created,nextcheck);
+      /* Only show marginals, completes, and cert_depth in the classic
+	 or PGP trust models since they are not meaningful
+	 otherwise. */
+
+      if(trust_model==TM_PGP || trust_model==TM_CLASSIC)
+	printf(":%d:%d:%d",marginals,completes,cert_depth);
+
+      printf("\n");
     }
 
   if( !list )
@@ -94,42 +111,177 @@ secret_key_list( STRLIST list )
 }
 
 void
-show_policy_url(PKT_signature *sig,int indent)
+print_seckey_info (PKT_secret_key *sk)
+{
+  u32 keyid[2];
+  char *p;
+
+  keyid_from_sk (sk, keyid);
+  p=get_user_id_native(keyid);
+
+  tty_printf ("\nsec  %4u%c/%s %s %s\n",
+	      nbits_from_sk (sk),
+	      pubkey_letter (sk->pubkey_algo),
+	      keystr(keyid), datestr_from_sk (sk), p);
+    
+  m_free (p);
+}
+
+/* Print information about the public key.  With FP passed as NULL,
+   the tty output interface is used, otherwise output is directted to
+   the given stream. */
+void
+print_pubkey_info (FILE *fp, PKT_public_key *pk)
+{
+  u32 keyid[2];
+  char *p;
+
+  keyid_from_pk (pk, keyid);
+
+  /* If the pk was chosen by a particular user ID, that is the one to
+     print. */
+  if(pk->user_id)
+    p=utf8_to_native(pk->user_id->name,pk->user_id->len,0);
+  else
+    p=get_user_id_native(keyid);
+
+  if (fp)
+    fprintf (fp, "pub  %4u%c/%s %s %s\n",
+             nbits_from_pk (pk),
+             pubkey_letter (pk->pubkey_algo),
+             keystr(keyid), datestr_from_pk (pk), p);
+  else
+    tty_printf ("\npub  %4u%c/%s %s %s\n",
+                nbits_from_pk (pk), pubkey_letter (pk->pubkey_algo),
+                keystr(keyid), datestr_from_pk (pk), p);
+
+  m_free (p);
+}
+
+/* Flags = 0x01 hashed 0x02 critical */
+static void
+status_one_subpacket(sigsubpkttype_t type,size_t len,int flags,const byte *buf)
+{
+  char status[40];
+
+  /* Don't print these. */
+  if(len>256)
+    return;
+
+  sprintf(status,"%d %u %u ",type,flags,(unsigned int)len);
+
+  write_status_text_and_buffer(STATUS_SIG_SUBPACKET,status,buf,len,0);
+}
+
+/*
+  mode=0 for stdout.
+  mode=1 for log_info + status messages
+  mode=2 for status messages only
+*/
+
+void
+show_policy_url(PKT_signature *sig,int indent,int mode)
 {
   const byte *p;
   size_t len;
   int seq=0,crit;
+  FILE *fp=mode?log_stream():stdout;
 
   while((p=enum_sig_subpkt(sig->hashed,SIGSUBPKT_POLICY,&len,&seq,&crit)))
     {
-      int i;
+      if(mode!=2)
+	{
+	  int i;
+	  const char *str;
 
-      for(i=0;i<indent;i++)
-	putchar(' ');
+	  for(i=0;i<indent;i++)
+	    putchar(' ');
 
-      /* This isn't UTF8 as it is a URL(?) */
-      if(crit)
-	printf(_("Critical signature policy: "));
-      else
-	printf(_("Signature policy: "));
-      print_string(stdout,p,len,0);
-      printf("\n");
+	  if(crit)
+	    str=_("Critical signature policy: ");
+	  else
+	    str=_("Signature policy: ");
+	  if(mode)
+	    log_info("%s",str);
+	  else
+	    printf("%s",str);
+	  print_utf8_string(fp,p,len);
+	  fprintf(fp,"\n");
+	}
+
+      if(mode)
+	write_status_buffer ( STATUS_POLICY_URL, p, len, 0 );
     }
 }
 
+/*
+  mode=0 for stdout.
+  mode=1 for log_info + status messages
+  mode=2 for status messages only
+*/
+/* TODO: use this */
 void
-show_notation(PKT_signature *sig,int indent)
+show_keyserver_url(PKT_signature *sig,int indent,int mode)
 {
   const byte *p;
   size_t len;
   int seq=0,crit;
+  FILE *fp=mode?log_stream():stdout;
+
+  while((p=enum_sig_subpkt(sig->hashed,SIGSUBPKT_PREF_KS,&len,&seq,&crit)))
+    {
+      if(mode!=2)
+	{
+	  int i;
+	  const char *str;
+
+	  for(i=0;i<indent;i++)
+	    putchar(' ');
+
+	  if(crit)
+	    str=_("Critical preferred keyserver: ");
+	  else
+	    str=_("Preferred keyserver: ");
+	  if(mode)
+	    log_info("%s",str);
+	  else
+	    printf("%s",str);
+	  print_utf8_string(fp,p,len);
+	  fprintf(fp,"\n");
+	}
+
+      if(mode)
+	status_one_subpacket(SIGSUBPKT_PREF_KS,len,(crit?0x02:0)|0x01,p);
+    }
+}
+
+/*
+  mode=0 for stdout.
+  mode=1 for log_info + status messages
+  mode=2 for status messages only
+
+  which bits:
+  1 == standard notations
+  2 == user notations
+*/
+
+void
+show_notation(PKT_signature *sig,int indent,int mode,int which)
+{
+  const byte *p;
+  size_t len;
+  int seq=0,crit;
+  FILE *fp=mode?log_stream():stdout;
+
+  if(which==0)
+    which=3;
 
   /* There may be multiple notations in the same sig. */
 
   while((p=enum_sig_subpkt(sig->hashed,SIGSUBPKT_NOTATION,&len,&seq,&crit)))
     if(len>=8)
       {
-	int n1,n2,i;
+	int n1,n2;
 
 	n1=(p[4]<<8)|p[5];
 	n2=(p[6]<<8)|p[7];
@@ -137,26 +289,47 @@ show_notation(PKT_signature *sig,int indent)
 	if(8+n1+n2!=len)
 	  {
 	    log_info(_("WARNING: invalid notation data found\n"));
-	    return;
+	    continue;
 	  }
 
-	for(i=0;i<indent;i++)
-	  putchar(' ');
+	if(mode!=2)
+	  {
+	    int has_at=!!memchr(p+8,'@',n1);
 
-	/* This is UTF8 */
-	if(crit)
-	  printf(_("Critical signature notation: "));
-	else
-	  printf(_("Signature notation: "));
-	print_utf8_string(stdout,p+8,n1);
-	printf("=");
+	    if((which&1 && !has_at) || (which&2 && has_at))
+	      {
+		int i;
+		const char *str;
 
-	if(*p&0x80)
-	  print_utf8_string(stdout,p+8+n1,n2);
-	else
-	  printf("[ %s ]",_("not human readable"));
+		for(i=0;i<indent;i++)
+		  putchar(' ');
 
-	printf("\n");
+		/* This is UTF8 */
+		if(crit)
+		  str=_("Critical signature notation: ");
+		else
+		  str=_("Signature notation: ");
+		if(mode)
+		  log_info("%s",str);
+		else
+		  printf("%s",str);
+		print_utf8_string(fp,p+8,n1);
+		fprintf(fp,"=");
+
+		if(*p&0x80)
+		  print_utf8_string(fp,p+8+n1,n2);
+		else
+		  fprintf(fp,"[ %s ]",_("not human readable"));
+
+		fprintf(fp,"\n");
+	      }
+	  }
+
+	if(mode)
+	  {
+	    write_status_buffer ( STATUS_NOTATION_NAME, p+8   , n1, 0 );
+	    write_status_buffer ( STATUS_NOTATION_DATA, p+8+n1, n2, 50 );
+	  }
       }
   else
     log_info(_("WARNING: invalid notation data found\n"));
@@ -247,7 +420,7 @@ list_one( STRLIST names, int secret )
     KBNODE keyblock = NULL;
     GETKEY_CTX ctx;
     const char *resname;
-    char *keyring_str = _("Keyring");
+    const char *keyring_str = _("Keyring");
     int i;
     struct sig_stats stats;
 
@@ -270,7 +443,7 @@ list_one( STRLIST names, int secret )
 	    return;
 	}
 	do {
-	    if (opt.show_keyring && !opt.with_colons) {
+	    if ((opt.list_options&LIST_SHOW_KEYRING) && !opt.with_colons) {
 		resname = keydb_get_resource_name (get_ctx_handle(ctx));
 		printf("%s: %s\n", keyring_str, resname);
 		for(i = strlen(resname) + strlen(keyring_str) + 2; i; i-- )
@@ -290,7 +463,7 @@ list_one( STRLIST names, int secret )
 	    return;
 	}
 	do {
-	  if (opt.show_keyring && !opt.with_colons) {
+	  if ((opt.list_options&LIST_SHOW_KEYRING) && !opt.with_colons) {
 		resname = keydb_get_resource_name (get_ctx_handle(ctx));
 		printf("%s: %s\n", keyring_str, resname);
 		for(i = strlen(resname) + strlen(keyring_str) + 2; i; i-- )
@@ -309,7 +482,7 @@ list_one( STRLIST names, int secret )
 }
 
 static void
-print_key_data( PKT_public_key *pk, u32 *keyid )
+print_key_data( PKT_public_key *pk )
 {
     int n = pk ? pubkey_get_npkey( pk->pubkey_algo ) : 0;
     int i;
@@ -338,11 +511,14 @@ print_capabilities (PKT_public_key *pk, PKT_secret_key *sk, KBNODE keyblock)
 	  if( pk? pk->is_primary : sk->is_primary )
 	    putchar ('c');
 	}
+
+      if ( (use & PUBKEY_USAGE_AUTH) )
+        putchar ('a');
     }
 
     if ( keyblock ) { /* figure out the usable capabilities */
         KBNODE k;
-        int enc=0, sign=0, cert=0, disabled=0;
+        int enc=0, sign=0, cert=0, auth=0, disabled=0;
 
         for (k=keyblock; k; k = k->next ) {
             if ( k->pkt->pkttype == PKT_PUBLIC_KEY 
@@ -361,6 +537,8 @@ print_capabilities (PKT_public_key *pk, PKT_secret_key *sk, KBNODE keyblock)
 			if(pk->is_primary)
 			  cert = 1;
 		      }
+                    if ( (pk->pubkey_usage & PUBKEY_USAGE_AUTH) )
+                      auth = 1;
                 }
             }
             else if ( k->pkt->pkttype == PKT_SECRET_KEY 
@@ -376,6 +554,8 @@ print_capabilities (PKT_public_key *pk, PKT_secret_key *sk, KBNODE keyblock)
 			if(sk->is_primary)
 			  cert = 1;
 		      }
+                    if ( (sk->pubkey_usage & PUBKEY_USAGE_AUTH) )
+                        auth = 1;
                 }
             }
         }
@@ -385,6 +565,8 @@ print_capabilities (PKT_public_key *pk, PKT_secret_key *sk, KBNODE keyblock)
             putchar ('S');
         if (cert)
             putchar ('C');
+        if (auth)
+            putchar ('A');
         if (disabled)
             putchar ('D');
     }
@@ -392,6 +574,51 @@ print_capabilities (PKT_public_key *pk, PKT_secret_key *sk, KBNODE keyblock)
     putchar(':');
 }
 
+/* Flags = 0x01 hashed 0x02 critical */
+static void
+print_one_subpacket(sigsubpkttype_t type,size_t len,int flags,const byte *buf)
+{
+  size_t i;
+
+  printf("spk:%d:%u:%u:",type,flags,(unsigned int)len);
+
+  for(i=0;i<len;i++)
+    {
+      /* printable ascii other than : and % */
+      if(buf[i]>=32 && buf[i]<=126 && buf[i]!=':' && buf[i]!='%')
+	printf("%c",buf[i]);
+      else
+	printf("%%%02X",buf[i]);
+    }
+
+  printf("\n");
+}
+
+void
+print_subpackets_colon(PKT_signature *sig)
+{
+  byte *i;
+
+  assert(opt.show_subpackets);
+
+  for(i=opt.show_subpackets;*i;i++)
+    {
+      const byte *p;
+      size_t len;
+      int seq,crit;
+
+      seq=0;
+
+      while((p=enum_sig_subpkt(sig->hashed,*i,&len,&seq,&crit)))
+	print_one_subpacket(*i,len,0x01|(crit?0x02:0),p);
+
+      seq=0;
+
+      while((p=enum_sig_subpkt(sig->unhashed,*i,&len,&seq,&crit)))
+	print_one_subpacket(*i,len,0x00|(crit?0x02:0),p);
+    }
+}
+
 void
 dump_attribs(const PKT_user_id *uid,PKT_public_key *pk,PKT_secret_key *sk)
 {
@@ -440,9 +667,8 @@ list_keyblock_print ( KBNODE keyblock, int secret, int fpr, void *opaque )
     KBNODE node;
     PKT_public_key *pk;
     PKT_secret_key *sk;
-    u32 keyid[2];
-    int any=0;
     struct sig_stats *stats=opaque;
+    int skip_sigs=0;
 
     /* get the keyid from the keyblock */
     node = find_kbnode( keyblock, secret? PKT_SECRET_KEY : PKT_PUBLIC_KEY );
@@ -452,106 +678,188 @@ list_keyblock_print ( KBNODE keyblock, int secret, int fpr, void *opaque )
 	return;
     }
 
-    if( secret ) {
+    if( secret )
+      {
 	pk = NULL;
 	sk = node->pkt->pkt.secret_key;
-	keyid_from_sk( sk, keyid );
-        printf("sec%c %4u%c/%08lX %s ", (sk->protect.s2k.mode==1001)?'#':' ',
-	                                nbits_from_sk( sk ),
-				        pubkey_letter( sk->pubkey_algo ),
-				        (ulong)keyid[1],
-				        datestr_from_sk( sk ) );
-    }
-    else {
+
+        printf("sec%c  %4u%c/%s %s",(sk->protect.s2k.mode==1001)?'#':
+	       (sk->protect.s2k.mode==1002)?'>':' ',
+	       nbits_from_sk( sk ),pubkey_letter( sk->pubkey_algo ),
+	       keystr_from_sk(sk),datestr_from_sk( sk ));
+
+	if(sk->has_expired)
+	  {
+	    printf(" [");
+	    printf(_("expired: %s"),expirestr_from_sk(sk));
+	    printf("]");
+	  }
+	else if(sk->expiredate )
+	  {
+	    printf(" [");
+	    printf(_("expires: %s"),expirestr_from_sk(sk));
+	    printf("]");
+	  }
+
+	printf("\n");
+      }
+    else
+      {
 	pk = node->pkt->pkt.public_key;
 	sk = NULL;
-	keyid_from_pk( pk, keyid );
-        printf("pub  %4u%c/%08lX %s ", nbits_from_pk( pk ),
-				       pubkey_letter( pk->pubkey_algo ),
-				       (ulong)keyid[1],
-				       datestr_from_pk( pk ) );
-    }
+
+	check_trustdb_stale();
+
+	printf("pub   %4u%c/%s %s",
+	       nbits_from_pk(pk),pubkey_letter(pk->pubkey_algo),
+	       keystr_from_pk(pk),datestr_from_pk( pk ));
+
+	/* We didn't include this before in the key listing, but there
+	   is room in the new format, so why not? */
+
+	if(pk->is_revoked)
+	  {
+	    printf(" [");
+	    printf(_("revoked: %s"),revokestr_from_pk(pk));
+	    printf("]");
+	  }
+	else if(pk->has_expired)
+	  {
+	    printf(" [");
+	    printf(_("expired: %s)"),expirestr_from_pk(pk));
+	    printf("]");
+	  }
+	else if(pk->expiredate)
+	  {
+	    printf(" [");
+	    printf(_("expires: %s"),expirestr_from_pk(pk));
+	    printf("]");
+	  }
+
+#if 0
+	/* I need to think about this some more.  It's easy enough to
+	   include, but it looks sort of confusing in the
+	   listing... */
+	if(opt.list_options&LIST_SHOW_VALIDITY)
+	  {
+	    int validity=get_validity(pk,NULL);
+	    printf(" [%s]",trust_value_to_string(validity));
+	  }
+#endif
+
+	printf("\n");
+      }
+
+    if( fpr )
+      print_fingerprint( pk, sk, 0 );
+    print_card_serialno (sk);
+    if( opt.with_key_data )
+      print_key_data( pk );
 
     for( kbctx=NULL; (node=walk_kbnode( keyblock, &kbctx, 0)) ; ) {
 	if( node->pkt->pkttype == PKT_USER_ID && !opt.fast_list_mode ) {
-            /* don't list revoked or expired UIDS unless we are in
-             * verbose mode and signature listing has not been
-             * requested */
-            if ( !opt.verbose && !opt.list_sigs &&
-                 (node->pkt->pkt.user_id->is_revoked ||
-		  node->pkt->pkt.user_id->is_expired ))
-                continue; 
+	    PKT_user_id *uid=node->pkt->pkt.user_id;
 
-	    if(attrib_fp && node->pkt->pkt.user_id->attrib_data!=NULL)
-	      dump_attribs(node->pkt->pkt.user_id,pk,sk);
+	    if((uid->is_expired || uid->is_revoked)
+	       && !(opt.list_options&LIST_SHOW_UNUSABLE_UIDS))
+	      {
+		skip_sigs=1;
+		continue;
+	      }
+	    else
+	      skip_sigs=0;
 
-	    if( any ) 
-                printf("uid%*s", 28, "");
+	    if(attrib_fp && uid->attrib_data!=NULL)
+	      dump_attribs(uid,pk,sk);
 
-            if ( node->pkt->pkt.user_id->is_revoked )
-                fputs ("[revoked] ", stdout);
-            if ( node->pkt->pkt.user_id->is_expired )
-                fputs ("[expired] ", stdout);
-            print_utf8_string( stdout,  node->pkt->pkt.user_id->name,
-                               node->pkt->pkt.user_id->len );
+	    if((uid->is_revoked || uid->is_expired)
+	       || ((opt.list_options&LIST_SHOW_UID_VALIDITY) && pk))
+	      {
+		const char *validity;
+		int indent;
+
+		validity=uid_trust_string_fixed(pk,uid);
+		indent=(keystrlen()+9)-atoi(uid_trust_string_fixed(NULL,NULL));
+
+		if(indent<0 || indent>40)
+		  indent=0;
+
+		printf("uid%*s%s ",indent,"",validity);
+	      }
+	    else
+	      printf("uid%*s",keystrlen()+10,"");
+
+            print_utf8_string( stdout, uid->name, uid->len );
 	    putchar('\n');
-	    if( !any ) {
-		if( fpr )
-		    print_fingerprint( pk, sk, 0 );
-		if( opt.with_key_data )
-		    print_key_data( pk, keyid );
-		any = 1;
-	    }
 
-	    if(opt.show_photos && node->pkt->pkt.user_id->attribs!=NULL)
-	      show_photos(node->pkt->pkt.user_id->attribs,
-			  node->pkt->pkt.user_id->numattribs,pk,sk);
+	    if((opt.list_options&LIST_SHOW_PHOTOS) && uid->attribs!=NULL)
+	      show_photos(uid->attribs,uid->numattribs,pk,sk);
 	}
-	else if( node->pkt->pkttype == PKT_PUBLIC_SUBKEY ) {
-	    u32 keyid2[2];
+	else if( node->pkt->pkttype == PKT_PUBLIC_SUBKEY )
+	  {
 	    PKT_public_key *pk2 = node->pkt->pkt.public_key;
 
-	    if( !any ) {
-		putchar('\n');
-		if( fpr )
-		    print_fingerprint( pk, sk, 0 ); /* of the main key */
-		any = 1;
-	    }
+	    if((pk2->is_revoked || pk2->has_expired)
+	       && !(opt.list_options&LIST_SHOW_UNUSABLE_SUBKEYS))
+	      {
+		skip_sigs=1;
+		continue;
+	      }
+	    else
+	      skip_sigs=0;
 
-	    keyid_from_pk( pk2, keyid2 );
-            printf("sub  %4u%c/%08lX %s", nbits_from_pk( pk2 ),
-                   pubkey_letter( pk2->pubkey_algo ),
-                   (ulong)keyid2[1],
-                   datestr_from_pk( pk2 ) );
-            if( pk2->expiredate ) {
-                printf(_(" [expires: %s]"), expirestr_from_pk( pk2 ) );
-            }
+            printf("sub   %4u%c/%s %s",
+		   nbits_from_pk( pk2 ),pubkey_letter( pk2->pubkey_algo ),
+		   keystr_from_pk(pk2),datestr_from_pk(pk2));
+	    if( pk2->is_revoked )
+	      {
+		printf(" [");
+		printf(_("revoked: %s"),revokestr_from_pk(pk2));
+		printf("]");
+	      }
+	    else if( pk2->has_expired )
+	      {
+		printf(" [");
+		printf(_("expired: %s"),expirestr_from_pk(pk2));
+		printf("]");
+	      }
+	    else if( pk2->expiredate )
+	      {
+		printf(" [");
+		printf(_("expires: %s"),expirestr_from_pk(pk2));
+		printf("]");
+	      }
             putchar('\n');
 	    if( fpr > 1 )
-		print_fingerprint( pk2, NULL, 0 );
+	      print_fingerprint( pk2, NULL, 0 );
 	    if( opt.with_key_data )
-		print_key_data( pk2, keyid2 );
-	}
-	else if( node->pkt->pkttype == PKT_SECRET_SUBKEY ) {
-	    u32 keyid2[2];
+	      print_key_data( pk2 );
+	  }
+	else if( node->pkt->pkttype == PKT_SECRET_SUBKEY )
+	  {
 	    PKT_secret_key *sk2 = node->pkt->pkt.secret_key;
 
-	    if( !any ) {
-		putchar('\n');
-		if( fpr )
-		    print_fingerprint( pk, sk, 0 ); /* of the main key */
-		any = 1;
-	    }
-
-	    keyid_from_sk( sk2, keyid2 );
-            printf("ssb  %4u%c/%08lX %s\n", nbits_from_sk( sk2 ),
-					   pubkey_letter( sk2->pubkey_algo ),
-					   (ulong)keyid2[1],
-					   datestr_from_sk( sk2 ) );
+            printf("ssb%c  %4u%c/%s %s",
+                   (sk2->protect.s2k.mode==1001)?'#':
+                   (sk2->protect.s2k.mode==1002)?'>':' ',
+		   nbits_from_sk( sk2 ),pubkey_letter( sk2->pubkey_algo ),
+		   keystr_from_sk(sk2),datestr_from_sk( sk2 ) );
+            if( sk2->expiredate )
+	      {
+		printf(" [");
+		printf(_("expires: %s"),expirestr_from_sk(sk2));
+		printf("]");
+	      }
+	    putchar('\n');
 	    if( fpr > 1 )
-		print_fingerprint( NULL, sk2, 0 );
-	}
-	else if( opt.list_sigs && node->pkt->pkttype == PKT_SIGNATURE ) {
+              {
+                print_fingerprint( NULL, sk2, 0 );
+                print_card_serialno (sk2);
+              }
+	  }
+	else if( opt.list_sigs
+		 && node->pkt->pkttype == PKT_SIGNATURE
+		 && !skip_sigs ) {
 	    PKT_signature *sig = node->pkt->pkt.signature;
 	    int sigrc;
             char *sigstr;
@@ -570,31 +878,12 @@ list_keyblock_print ( KBNODE keyblock, int secret, int fpr, void *opaque )
 		/* TODO: Make sure a cached sig record here still has
                    the pk that issued it.  See also
                    keyedit.c:print_and_check_one_sig */
-
 	    }
 	    else {
 		rc = 0;
 		sigrc = ' ';
 	    }
 
-	    if( !any ) { /* no user id, (maybe a revocation follows)*/
-	      /* Check if the pk is really revoked - there could be a
-                 0x20 sig packet there even if we are not revoked
-                 (say, if a revocation key issued the packet, but the
-                 revocation key isn't present to verify it.) */
-		if( sig->sig_class == 0x20 && pk->is_revoked )
-		    puts("[revoked]");
-		else if( sig->sig_class == 0x18 )
-		    puts("[key binding]");
-		else if( sig->sig_class == 0x28 )
-		    puts("[subkey revoked]");
-		else
-		    putchar('\n');
-		if( fpr )
-		    print_fingerprint( pk, sk, 0 );
-		any=1;
-	    }
-
 	    if( sig->sig_class == 0x20 || sig->sig_class == 0x28
 				       || sig->sig_class == 0x30 )
 	       sigstr = "rev";
@@ -611,7 +900,7 @@ list_keyblock_print ( KBNODE keyblock, int secret, int fpr, void *opaque )
 	    }
 
             fputs( sigstr, stdout );
-	    printf("%c%c %c%c%c%c%c %08lX %s   ",
+	    printf("%c%c %c%c%c%c%c%c %s %s",
                    sigrc,(sig->sig_class-0x10>0 &&
                           sig->sig_class-0x10<4)?'0'+sig->sig_class-0x10:' ',
                    sig->flags.exportable?' ':'L',
@@ -619,7 +908,12 @@ list_keyblock_print ( KBNODE keyblock, int secret, int fpr, void *opaque )
                    sig->flags.policy_url?'P':' ',
                    sig->flags.notation?'N':' ',
                    sig->flags.expired?'X':' ',
-                   (ulong)sig->keyid[1], datestr_from_sig(sig));
+		   (sig->trust_depth>9)?'T':
+		   (sig->trust_depth>0)?'0'+sig->trust_depth:' ',
+		   keystr(sig->keyid),datestr_from_sig(sig));
+	    if(opt.list_options&LIST_SHOW_SIG_EXPIRE)
+	      printf(" %s", expirestr_from_sig(sig));
+	    printf("  ");
 	    if( sigrc == '%' )
 		printf("[%s] ", g10_errstr(rc) );
 	    else if( sigrc == '?' )
@@ -632,11 +926,18 @@ list_keyblock_print ( KBNODE keyblock, int secret, int fpr, void *opaque )
 	    }
 	    putchar('\n');
 
-	    if(sig->flags.policy_url && opt.show_policy_url)
-	      show_policy_url(sig,3);
+	    if(sig->flags.policy_url
+	       && (opt.list_options&LIST_SHOW_POLICY_URLS))
+	      show_policy_url(sig,3,0);
 
-	    if(sig->flags.notation && opt.show_notation)
-	      show_notation(sig,3);
+	    if(sig->flags.notation && (opt.list_options&LIST_SHOW_NOTATIONS))
+	      show_notation(sig,3,0,
+			    ((opt.list_options&LIST_SHOW_STD_NOTATIONS)?1:0)+
+			    ((opt.list_options&LIST_SHOW_USER_NOTATIONS)?2:0));
+
+	    if(sig->flags.pref_ks
+	       && (opt.list_options&LIST_SHOW_KEYSERVER_URLS))
+	      show_keyserver_url(sig,3,0);
 
 	    /* fixme: check or list other sigs here */
 	}
@@ -657,6 +958,7 @@ list_keyblock_colon( KBNODE keyblock, int secret, int fpr )
     int any=0;
     int trustletter = 0;
     int ulti_hack = 0;
+    int i;
 
     /* get the keyid from the keyblock */
     node = find_kbnode( keyblock, secret? PKT_SECRET_KEY : PKT_PUBLIC_KEY );
@@ -697,15 +999,12 @@ list_keyblock_colon( KBNODE keyblock, int secret, int fpr )
                 ulti_hack = 1;
             putchar(trustletter);
         }
-        printf(":%u:%d:%08lX%08lX:%s:%s:",
+        printf(":%u:%d:%08lX%08lX:%s:%s::",
 		    nbits_from_pk( pk ),
 		    pk->pubkey_algo,
 		    (ulong)keyid[0],(ulong)keyid[1],
 		    colon_datestr_from_pk( pk ),
 		    colon_strtime (pk->expiredate) );
-        if( pk->local_id )
-            printf("%lu", pk->local_id );
-        putchar(':');
         if( !opt.fast_list_mode && !opt.no_expensive_trust_checks  )
             putchar( get_ownertrust_info(pk) );
 	    putchar(':');
@@ -716,53 +1015,74 @@ list_keyblock_colon( KBNODE keyblock, int secret, int fpr )
         putchar(':');
         putchar(':');
         print_capabilities (pk, sk, keyblock);
+        if (secret) {
+          putchar(':'); /* End of field 13. */
+          putchar(':'); /* End of field 14. */
+          if (sk->protect.s2k.mode == 1001)
+            putchar('#'); /* Key is just a stub. */
+          else if (sk->protect.s2k.mode == 1002) {
+            /* Key is stored on an external token (card) or handled by
+               the gpg-agent.  Print the serial number of that token
+               here. */
+            for (i=0; i < sk->protect.ivlen; i++)
+              printf ("%02X", sk->protect.iv[i]);
+          }
+          putchar(':'); /* End of field 15. */
+        }
         putchar('\n');
         if( fpr )
             print_fingerprint( pk, sk, 0 );
         if( opt.with_key_data )
-            print_key_data( pk, keyid );
+            print_key_data( pk );
         any = 1;
     }
 
 
     for( kbctx=NULL; (node=walk_kbnode( keyblock, &kbctx, 0)) ; ) {
 	if( node->pkt->pkttype == PKT_USER_ID && !opt.fast_list_mode ) {
+	    PKT_user_id *uid=node->pkt->pkt.user_id;
 	    if(attrib_fp && node->pkt->pkt.user_id->attrib_data!=NULL)
 	      dump_attribs(node->pkt->pkt.user_id,pk,sk);
             /*
              * Fixme: We need a is_valid flag here too 
              */
 	    if( any ) {
-	        char *str=node->pkt->pkt.user_id->attrib_data?"uat":"uid";
+	        char *str=uid->attrib_data?"uat":"uid";
 		/* If we're listing a secret key, leave out the
 		   validity values for now.  This is handled better in
 		   1.9. */
 		if ( sk )
-		    printf("%s:::::::::",str);
-		else if ( node->pkt->pkt.user_id->is_revoked )
-        	    printf("%s:r::::::::",str);
-                else if ( node->pkt->pkt.user_id->is_expired )
-        	    printf("%s:e::::::::",str);
+        	    printf("%s:::::",str);
+                else if ( uid->is_revoked )
+        	    printf("%s:r::::",str);
+                else if ( uid->is_expired )
+        	    printf("%s:e::::",str);
 		else if ( opt.no_expensive_trust_checks )
-        	    printf("%s:::::::::",str);
+        	    printf("%s:::::",str);
                 else {
 		    int uid_validity;
 
 		    if( pk && !ulti_hack )
-		      uid_validity=get_validity_info (pk,
-						      node->pkt->pkt.user_id);
+		      uid_validity=get_validity_info (pk, uid);
 		    else
 			uid_validity = 'u';
-		    printf("%s:%c::::::::",str,uid_validity);
+		    printf("%s:%c::::",str,uid_validity);
                 }
+
+		printf("%s:",colon_strtime(uid->created));
+		printf("%s:",colon_strtime(uid->expiredate));
+
+		namehash_from_uid(uid);
+
+		for(i=0; i < 20; i++ )
+		  printf("%02X",uid->namehash[i]);
+
+		printf("::");
 	    }
-	    if(node->pkt->pkt.user_id->attrib_data)
-	      printf("%u %lu",
-		     node->pkt->pkt.user_id->numattribs,
-		     node->pkt->pkt.user_id->attrib_len);
+	    if(uid->attrib_data)
+	      printf("%u %lu",uid->numattribs,uid->attrib_len);
             else
-	      print_string( stdout,  node->pkt->pkt.user_id->name,
-			    node->pkt->pkt.user_id->len, ':' );
+	      print_string(stdout,uid->name,uid->len, ':' );
             putchar(':');
 	    if (any)
                 putchar('\n');
@@ -773,7 +1093,7 @@ list_keyblock_colon( KBNODE keyblock, int secret, int fpr )
 		if( fpr )
 		    print_fingerprint( pk, sk, 0 );
 		if( opt.with_key_data )
-		    print_key_data( pk, keyid );
+		    print_key_data( pk );
 		any = 1;
 	    }
 	}
@@ -806,7 +1126,7 @@ list_keyblock_colon( KBNODE keyblock, int secret, int fpr )
 	        if(trustletter)
 		  printf("%c", trustletter );
             }
-            printf(":%u:%d:%08lX%08lX:%s:%s:",
+            printf(":%u:%d:%08lX%08lX:%s:%s:::::",
 			nbits_from_pk( pk2 ),
 			pk2->pubkey_algo,
 			(ulong)keyid2[0],(ulong)keyid2[1],
@@ -814,18 +1134,12 @@ list_keyblock_colon( KBNODE keyblock, int secret, int fpr )
 			colon_strtime (pk2->expiredate)
 			/* fixme: add LID and ownertrust here */
 						);
-            if( pk->local_id ) /* use the local_id of the main key??? */
-                printf("%lu", pk->local_id );
-            putchar(':');
-            putchar(':');
-            putchar(':');
-            putchar(':');
             print_capabilities (pk2, NULL, NULL);
             putchar('\n');
 	    if( fpr > 1 )
 		print_fingerprint( pk2, NULL, 0 );
 	    if( opt.with_key_data )
-		print_key_data( pk2, keyid2 );
+		print_key_data( pk2 );
 	}
 	else if( node->pkt->pkttype == PKT_SECRET_SUBKEY ) {
 	    u32 keyid2[2];
@@ -850,14 +1164,34 @@ list_keyblock_colon( KBNODE keyblock, int secret, int fpr )
 			colon_strtime (sk2->expiredate)
                    /* fixme: add LID */ );
             print_capabilities (NULL, sk2, NULL);
+            if (opt.fixed_list_mode) {
+              /* We print the serial number only in fixed list mode
+                 for the primary key so, so avoid questions we print
+                 it for subkeys also only in this mode.  There is no
+                 technical reason, though. */
+              putchar(':'); /* End of field 13. */
+              putchar(':'); /* End of field 14. */
+              if (sk2->protect.s2k.mode == 1001)
+                putchar('#'); /* Key is just a stub. */
+              else if (sk2->protect.s2k.mode == 1002) {
+                /* Key is stored on an external token (card) or handled by
+                   the gpg-agent.  Print the serial number of that token
+                   here. */
+                for (i=0; i < sk2->protect.ivlen; i++)
+                  printf ("%02X", sk2->protect.iv[i]);
+              }
+              putchar(':'); /* End of field 15. */
+            }
             putchar ('\n');
 	    if( fpr > 1 )
-		print_fingerprint( NULL, sk2, 0 );
+              print_fingerprint( NULL, sk2, 0 );
 	}
 	else if( opt.list_sigs && node->pkt->pkttype == PKT_SIGNATURE ) {
 	    PKT_signature *sig = node->pkt->pkt.signature;
-	    int sigrc;
+	    int sigrc,fprokay=0;
             char *sigstr;
+	    size_t fplen;
+	    byte fparray[MAX_FINGERPRINT_LEN];
 
 	    if( !any ) { /* no user id, (maybe a revocation follows)*/
 		if( sig->sig_class == 0x20 )
@@ -891,8 +1225,14 @@ list_keyblock_colon( KBNODE keyblock, int secret, int fpr )
 		continue;
 	    }
 	    if( opt.check_sigs ) {
+	        PKT_public_key *signer_pk=NULL;
+
 		fflush(stdout);
-		rc = check_key_signature( keyblock, node, NULL );
+		if(opt.no_sig_cache)
+		  signer_pk=m_alloc_clear(sizeof(PKT_public_key));
+
+		rc = check_key_signature2( keyblock, node, NULL, signer_pk,
+					   NULL, NULL, NULL );
 		switch( rc ) {
 		  case 0:		   sigrc = '!'; break;
 		  case G10ERR_BAD_SIGN:    sigrc = '-'; break;
@@ -900,6 +1240,16 @@ list_keyblock_colon( KBNODE keyblock, int secret, int fpr )
 		  case G10ERR_UNU_PUBKEY:  sigrc = '?'; break;
 		  default:		   sigrc = '%'; break;
 		}
+
+		if(opt.no_sig_cache)
+		  {
+		    if(rc==0)
+		      {
+			fingerprint_from_pk (signer_pk, fparray, &fplen);
+			fprokay=1;
+		      }
+		    free_public_key(signer_pk);
+		  }
 	    }
 	    else {
 		rc = 0;
@@ -909,10 +1259,20 @@ list_keyblock_colon( KBNODE keyblock, int secret, int fpr )
             putchar(':');
             if( sigrc != ' ' )
                 putchar(sigrc);
-            printf("::%d:%08lX%08lX:%s:%s:::", sig->pubkey_algo,
-						 (ulong)sig->keyid[0],
-			   (ulong)sig->keyid[1], colon_datestr_from_sig(sig),
-		           colon_expirestr_from_sig(sig));
+            printf("::%d:%08lX%08lX:%s:%s:", sig->pubkey_algo,
+		   (ulong)sig->keyid[0], (ulong)sig->keyid[1],
+		   colon_datestr_from_sig(sig),
+		   colon_expirestr_from_sig(sig));
+
+	    if(sig->trust_depth || sig->trust_value)
+	      printf("%d %d",sig->trust_depth,sig->trust_value);
+	    printf(":");
+
+	    if(sig->trust_regexp)
+	      print_string(stdout,sig->trust_regexp,
+			   strlen(sig->trust_regexp),':');
+	    printf(":");
+
 	    if( sigrc == '%' )
 		printf("[%s] ", g10_errstr(rc) );
 	    else if( sigrc == '?' )
@@ -923,7 +1283,23 @@ list_keyblock_colon( KBNODE keyblock, int secret, int fpr )
                 print_string( stdout, p, n, ':' );
 		m_free(p);
 	    }
-            printf(":%02x%c:\n", sig->sig_class,sig->flags.exportable?'x':'l');
+            printf(":%02x%c:", sig->sig_class,sig->flags.exportable?'x':'l');
+
+	    if(opt.no_sig_cache && opt.check_sigs && fprokay)
+	      {
+		printf(":");
+
+		for (i=0; i < fplen ; i++ )
+		  printf ("%02X", fparray[i] );
+
+		printf(":");
+	      }
+
+	    printf("\n");
+
+	    if(opt.show_subpackets)
+	      print_subpackets_colon(sig);
+
 	    /* fixme: check or list other sigs here */
 	}
     }
@@ -1061,11 +1437,11 @@ print_fingerprint (PKT_public_key *pk, PKT_secret_key *sk, int mode )
     }
     else if (mode == 3) {
         fp = NULL; /* use tty */
-	text = _("     Key fingerprint =");
+	text = _("      Key fingerprint =");
     }
     else {
         fp = stdout;
-	text = _("     Key fingerprint =");
+	text = _("      Key fingerprint =");
     }
   
     if (sk)
@@ -1119,6 +1495,41 @@ print_fingerprint (PKT_public_key *pk, PKT_secret_key *sk, int mode )
         tty_printf ("\n");
 }
 
+/* Print the serial number of an OpenPGP card if available. */
+static void
+print_card_serialno (PKT_secret_key *sk)
+{
+  int i;
+
+  if (!sk)
+    return;
+  if (!sk->is_protected || sk->protect.s2k.mode != 1002) 
+    return; /* Not a card. */
+  if (opt.with_colons)
+    return; /* Handled elsewhere. */
+
+  fputs (_("      Card serial no. ="), stdout);
+  putchar (' ');
+  if (sk->protect.ivlen == 16
+      && !memcmp (sk->protect.iv, "\xD2\x76\x00\x01\x24\x01", 6) )
+    { /* This is an OpenPGP card. Just print the relevant part. */
+      for (i=8; i < 14; i++)
+        {
+          if (i == 10)
+            putchar (' ');
+          printf ("%02X", sk->protect.iv[i]);
+        }
+    }
+  else
+    { /* Something is wrong: Print all. */
+      for (i=0; i < sk->protect.ivlen; i++)
+        printf ("%02X", sk->protect.iv[i]);
+    }
+  putchar ('\n');
+}
+
+
+
 void set_attrib_fd(int fd)
 {
   static int last_fd=-1;
@@ -1137,10 +1548,11 @@ void set_attrib_fd(int fd)
   else if( fd == 2 )
     attrib_fp = stderr;
   else
-    attrib_fp = fdopen( fd, "w" );
+    attrib_fp = fdopen( fd, "wb" );
   if( !attrib_fp ) {
     log_fatal("can't open fd %d for attribute output: %s\n",
 	      fd, strerror(errno));
   }
+
   last_fd = fd;
 }
