@@ -1,5 +1,6 @@
 /* cipher.h
- *	Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2003,
+ *               2004 Free Software Foundation, Inc.
  *
  * This file is part of GNUPG.
  *
@@ -31,17 +32,11 @@
 #define CIPHER_ALGO_3DES	 2
 #define CIPHER_ALGO_CAST5	 3
 #define CIPHER_ALGO_BLOWFISH	 4  /* blowfish 128 bit key */
-#define CIPHER_ALGO_SAFER_SK128  5
-#define CIPHER_ALGO_DES_SK	 6
+/* 5 & 6 are reserved */
 #define CIPHER_ALGO_AES          7
 #define CIPHER_ALGO_AES192       8
 #define CIPHER_ALGO_AES256       9
-#define CIPHER_ALGO_RIJNDAEL     CIPHER_ALGO_AES
-#define CIPHER_ALGO_RIJNDAEL192  CIPHER_ALGO_AES192
-#define CIPHER_ALGO_RIJNDAEL256  CIPHER_ALGO_AES256
-#define CIPHER_ALGO_TWOFISH	10  /* twofish 256 bit */
-#define CIPHER_ALGO_SKIPJACK   101  /* experimental: skipjack */
-#define CIPHER_ALGO_TWOFISH_OLD 102 /* experimental: twofish 128 bit */
+#define CIPHER_ALGO_TWOFISH 	10  /* twofish 256 bit */
 #define CIPHER_ALGO_DUMMY      110  /* no encryption at all */
 
 #define PUBKEY_ALGO_RSA        1
@@ -54,15 +49,26 @@
 #define PUBKEY_USAGE_SIG     1	    /* key is good for signatures */
 #define PUBKEY_USAGE_ENC     2	    /* key is good for encryption */
 #define PUBKEY_USAGE_CERT    4      /* key is also good to certify other keys*/
+#define PUBKEY_USAGE_AUTH    8      /* key is good for authentication */
+#define PUBKEY_USAGE_UNKNOWN 128    /* key has an unknown usage bit */
 
 #define DIGEST_ALGO_MD5       1
 #define DIGEST_ALGO_SHA1      2
 #define DIGEST_ALGO_RMD160    3
-#define DIGEST_ALGO_TIGER     6
+/* 4, 5, 6, and 7 are reserved */
+#define DIGEST_ALGO_SHA256    8
+#define DIGEST_ALGO_SHA384    9
+#define DIGEST_ALGO_SHA512   10
+
+#define COMPRESS_ALGO_NONE   0
+#define COMPRESS_ALGO_ZIP    1
+#define COMPRESS_ALGO_ZLIB   2
+#define COMPRESS_ALGO_BZIP2  3
 
 #define is_RSA(a)     ((a)==PUBKEY_ALGO_RSA || (a)==PUBKEY_ALGO_RSA_E \
 		       || (a)==PUBKEY_ALGO_RSA_S )
-#define is_ELGAMAL(a) ((a)==PUBKEY_ALGO_ELGAMAL || (a)==PUBKEY_ALGO_ELGAMAL_E)
+#define is_ELGAMAL(a) ((a)==PUBKEY_ALGO_ELGAMAL_E)
+#define is_DSA(a)     ((a)==PUBKEY_ALGO_DSA)
 
 typedef struct {
     int algo;
@@ -98,11 +104,11 @@ struct gcry_md_context {
 typedef struct gcry_md_context *MD_HANDLE;
 
 #ifndef EXTERN_UNLESS_MAIN_MODULE
- #if defined (__riscos__) && !defined (INCLUDED_BY_MAIN_MODULE)
-  #define EXTERN_UNLESS_MAIN_MODULE extern
- #else
-  #define EXTERN_UNLESS_MAIN_MODULE 
- #endif
+#if defined (__riscos__) && !defined (INCLUDED_BY_MAIN_MODULE)
+#define EXTERN_UNLESS_MAIN_MODULE extern
+#else
+#define EXTERN_UNLESS_MAIN_MODULE 
+#endif
 #endif
 EXTERN_UNLESS_MAIN_MODULE int g10c_debug_mode;
 EXTERN_UNLESS_MAIN_MODULE int g10_opt_verbose;
@@ -178,8 +184,7 @@ int pubkey_check_secret_key( int algo, MPI *skey );
 int pubkey_encrypt( int algo, MPI *resarr, MPI data, MPI *pkey );
 int pubkey_decrypt( int algo, MPI *result, MPI *data, MPI *skey );
 int pubkey_sign( int algo, MPI *resarr, MPI hash, MPI *skey );
-int pubkey_verify( int algo, MPI hash, MPI *data, MPI *pkey,
-		      int (*cmp)(void *, MPI), void *opaque );
+int pubkey_verify( int algo, MPI hash, MPI *data, MPI *pkey );
 
 /*-- smallprime.c --*/
 extern ushort small_prime_numbers[];
